/* Dumb NTP Server for TiVo 2.5.1
 *
 * Warren Toomey, wkt@tuhs.org
 *
 * BSD copyright license
 *
 * $Revision: 1.3 $
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <stdio.h>
#include <unistd.h>
#include <syslog.h>
#include <stdlib.h>
#include "ntp.h"

void convert_request_to_reply(struct pkt * P)
{
  u_char leap, version, mode;

  /* Set us up as same version as client, no leap year */
  version = PKT_VERSION(P->li_vn_mode);
  mode = MODE_SERVER;
  leap = 0;

  /* Say that we are an NTP stratum server */
  P->stratum = 3;
  P->li_vn_mode = PKT_LI_VN_MODE(leap, version, mode);

  /* Copy org timestamp over to xmt, reftime and rec */
  memcpy((void *)&(P->org), (void *)&(P->xmt), sizeof(l_fp));
  memcpy((void *)&(P->reftime), (void *)&(P->xmt), sizeof(l_fp));
  memcpy((void *)&(P->rec), (void *)&(P->xmt), sizeof(l_fp));

  /* Set a non-null refid, and a precision of -16 */
  P->refid = 0x010209c1;
  P->precision = -16;
}

int main()
{
  int sock;
  struct sockaddr_in name;
  struct sockaddr_in client;
  int client_len = sizeof(client);
  char buf[1024];

  /* Create socket from which to read. */
  sock = socket(AF_INET, SOCK_DGRAM, 0);
  if (sock < 0) { perror("opening datagram socket"); exit(1); }

  /* Create socket with wildcards. */
  name.sin_family = AF_INET;
  name.sin_addr.s_addr = INADDR_ANY;
  name.sin_port = htons(123);		/* NTP socket on UDP port 123 */

  /* Bind us to that port */
  if (bind(sock, (struct sockaddr *) & name, sizeof(name))) {
    perror("binding datagram socket"); exit(1);
  }

  /* Open the syslog, make ourselves a daemon */
  daemon(0,0); openlog("dumbntpd", 0, LOG_DAEMON);

  /* Act as a server forever */
  while (1) {
    /* Receive from the socket, ignore receive errors */
    if (recvfrom(sock, buf, 1024, 0, (struct sockaddr *) &client,
						&client_len) < 0) {
      syslog(LOG_INFO, "Failed to receive packet");
      continue;
    }

    /* Convert request to reply */
    convert_request_to_reply((struct pkt *) buf);
    syslog(LOG_INFO, "Got an NTP request, sending a reply back!");

    /* Send it back */
    sendto(sock, buf, sizeof(struct pkt), 0, (struct sockaddr *) & client,
	   client_len);
  }
  close(sock);
  exit(0);
}
